<?php
/**
 * @package     CSVI
 * @subpackage  J2Store
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - @year@ RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace j2store\com_j2store\model\export;

defined('_JEXEC') or die;

/**
 * Export J2Store Variants fields.
 *
 * @package     CSVI
 * @subpackage  J2Store
 * @since       7.7.0
 */
class Variant extends \CsviModelExports
{
	/**
	 * Export the data.
	 *
	 * @return  void.
	 *
	 * @since   7.7.0
	 */
	protected function exportBody()
	{
		if (parent::exportBody())
		{
			// Build something fancy to only get the fieldnames the user wants
			$exportFields = $this->fields->getFields();

			// Group by fields
			$groupFields   = json_decode($this->template->get('groupbyfields', '', 'string'));
			$groupBy       = [];
			$groupByFields = [];

			if (isset($groupFields->name))
			{
				$groupByFields = array_flip($groupFields->name);
			}

			// Sort selected fields
			$sortFields   = json_decode(
				$this->template->get('sortfields', '{}', 'string'),
				true,
				512,
				JSON_THROW_ON_ERROR
			);
			$sortBy       = [];
			$sortByFields = [];

			if (isset($sortFields->name, $sortFields->sortby))
			{
				foreach ($sortFields->sortby as $key => $sortName)
				{
					$sortByFields[$sortFields->name[$key]] = $sortName;
				}
			}

			// Fields which are needed for getting contents
			$userFields = [];

			foreach ($exportFields as $field)
			{
				$sortDirection = ($sortByFields[$field->field_name]) ?? 'ASC';

				switch ($field->field_name)
				{
					case 'parent_product_id':
					case 'parent_product_title':
					case 'parent_product_alias':
						$userFields[] = $this->db->quoteName('j2store_variants.product_id');

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('j2store_variants.product_id');
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('j2store_variants.product_id') . ' ' . $sortDirection;
						}
						break;
					case 'j2store_variant_id':
					case 'product_id':
					case 'is_master':
					case 'sku':
					case 'upc':
					case 'price':
					case 'pricing_calculator':
					case 'shipping':
					case 'params':
					case 'length':
					case 'width':
					case 'height':
					case 'length_class_id':
					case 'weight':
					case 'weight_class_id':
					case 'created_on':
					case 'created_by':
					case 'modified_on':
					case 'modified_by':
					case 'manage_stock':
					case 'quantity_restriction':
					case 'min_out_qty':
					case 'use_store_config_min_out_qty':
					case 'min_sale_qty':
					case 'use_store_config_min_sale_qty':
					case 'max_sale_qty':
					case 'use_store_config_max_sale_qty':
					case 'notify_qty':
					case 'use_store_config_notify_qty':
					case 'availability':
					case 'sold':
					case 'allow_backorder':
					case 'isdefault_variant':
						$userFields[] = $this->db->quoteName('j2store_variants.' . $field->field_name);

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('j2store_variants.' . $field->field_name);
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('j2store_variants.' . $field->field_name) . ' ' . $sortDirection;
						}
						break;
					case 'option_names':
					case 'option_types':
					case 'option_values':
						$userFields[] = $this->db->quoteName('j2store_variants.j2store_variant_id');

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('j2store_variants.j2store_variant_id');
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('j2store_variants.j2store_variant_id') . ' ' . $sortDirection;
						}
						break;
					case 'quantity':
						$userFields[] = $this->db->quoteName('j2store_productquantities.quantity');

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('j2store_productquantities.quantity');
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('j2store_productquantities.quantity') . ' ' . $sortDirection;
						}
						break;
					case 'custom':
						break;
					default:
						$userFields[] = $this->db->quoteName($field->field_name);

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName($field->field_name);
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName($field->field_name) . ' ' . $sortDirection;
						}

						break;
				}
			}

			// Build the query
			$userFields = array_unique($userFields);
			$query      = $this->db->getQuery(true);
			$query->select(implode(",\n", $userFields))
				->from($this->db->quoteName('#__j2store_variants', 'j2store_variants'))
				->leftJoin(
					$this->db->quoteName('#__j2store_products', 'j2store_products') . ' ON ' .
					$this->db->quoteName('j2store_products.j2store_product_id') . ' = ' . $this->db->quoteName('j2store_variants.product_id')
				)
				->leftJoin(
					$this->db->quoteName('#__j2store_productquantities', 'j2store_productquantities') . ' ON ' .
					$this->db->quoteName('j2store_productquantities.variant_id') . ' = ' . $this->db->quoteName('j2store_variants.j2store_variant_id')
				)
				->where($this->db->quoteName('j2store_variants.is_master') . ' = 0')
				->where($this->db->quoteName('j2store_products.product_type') . ' = ' . $this->db->quote('variable'));

			// Group the fields
			$groupBy = array_unique($groupBy);

			if (0 !== count($groupBy))
			{
				$query->group($groupBy);
			}

			// Sort set fields
			$sortBy = array_unique($sortBy);

			if (0 !== count($sortBy))
			{
				$query->order($sortBy);
			}

			// Add export limits
			$limits = $this->getExportLimit();

			// Execute the query
			$this->db->setQuery($query, $limits['offset'], $limits['limit']);
			$records = $this->db->getIterator();
			$this->log->add('Export query' . $query->__toString(), false);

			// Check if there are any records
			$logCount = $this->db->getNumRows();

			if ($logCount > 0)
			{
				foreach ($records as $record)
				{
					$this->log->incrementLinenumber();

					foreach ($exportFields as $field)
					{
						$fieldName  = $field->field_name;
						$fieldValue = '';

						// Set the field value
						if (isset($record->$fieldName))
						{
							$fieldValue = $record->$fieldName;
						}

						// Process the field
						switch ($fieldName)
						{
							case 'parent_product_id':
								$fieldValue = $record->product_id;
								break;
							case 'parent_product_title':
							case 'parent_product_alias':
								$selectField = str_replace('parent_product_', '', $fieldName);
								$query->clear()
									->select($this->db->quoteName('content.' . $selectField))
									->from($this->db->quoteName('#__content', 'content'))
									->leftJoin(
										$this->db->quoteName('#__j2store_products', 'products')
										. ' ON ' . $this->db->quoteName('products.product_source_id') . ' = ' . $this->db->quoteName('content.id')
									)
									->where($this->db->quoteName('products.j2store_product_id') . ' = ' . (int) $record->product_id);
								$this->db->setQuery($query);
								$fieldValue = $this->db->loadResult();
								break;
							case 'option_names':
							case 'option_types':
							case 'option_values':
								$variantId        = $record->j2store_variant_id;
								$fieldValue       = '';
								$newNameIds       = array();
								$namesList        = array();
								$optionValueNames = array();
								$typesList        = array();

								if ($variantId)
								{
									$query->clear()
										->select($this->db->quoteName('product_optionvalue_ids'))
										->from($this->db->quoteName('#__j2store_product_variant_optionvalues'))
										->where($this->db->quoteName('variant_id') . ' = ' . (int) $variantId);
									$this->db->setQuery($query);
									$optionValuesIds = $this->db->loadResult();

									if ($optionValuesIds)
									{
										$query->clear()
											->select(
												$this->db->quoteName(
													array(
														'optionvalues.option_id',
														'optionvalues.optionvalue_name',
														'product_optionvalues.productoption_id'
													)
												)
											)
											->from($this->db->quoteName('#__j2store_product_optionvalues', 'product_optionvalues'))
											->leftJoin(
												$this->db->quoteName('#__j2store_optionvalues', 'optionvalues')
												. ' ON ' . $this->db->quoteName('optionvalues.j2store_optionvalue_id')
												. ' = ' . $this->db->quoteName('product_optionvalues.optionvalue_id')
											)
											->where($this->db->quoteName('product_optionvalues.j2store_product_optionvalue_id') . ' IN (' . $optionValuesIds . ')');
										$this->db->setQuery($query);
										$optionNameIds = $this->db->loadObjectList();

										foreach ($optionNameIds as $nameIds)
										{
											$newNameIds[]       = $nameIds->option_id;
											$optionValueNames[] = $nameIds->optionvalue_name;
										}

										$nameIds = array_unique($newNameIds);

										if ($nameIds && $nameIds[0])
										{
											$query->clear()
												->select($this->db->quoteName('type'))
												->select($this->db->quoteName('option_name'))
												->select($this->db->quoteName('option_unique_name'))
												->from($this->db->quoteName('#__j2store_options'))
												->where($this->db->quoteName('j2store_option_id') . ' IN (' . implode(',', $nameIds) . ')');
											$this->db->setQuery($query);
											$nameAndTypeResult = $this->db->loadObjectList();

											foreach ($nameAndTypeResult as $result)
											{
												if ($result->option_name)
												{
													$namesList[] = $result->option_name;
												}
												else
												{
													$namesList[] = $result->option_unique_name;
												}

												$typesList[] = $result->type;
											}
										}

										if ($fieldName === 'option_names')
										{
											$fieldValue = implode('~', $namesList);
										}

										if ($fieldName === 'option_values')
										{
											$fieldValue = implode('~', array_unique($optionValueNames));
										}

										if ($fieldName === 'option_types')
										{
											$fieldValue = implode('~', $typesList);
										}
									}
								}

								break;
							default:
								break;
						}

						// Store the field value
						$this->fields->set($field->csvi_templatefield_id, $fieldValue);
					}

					// Output the data
					$this->addExportFields();

					// Output the contents
					$this->writeOutput();
				}
			}
			else
			{
				$this->addExportContent(\JText::_('COM_CSVI_NO_DATA_FOUND'));

				// Output the contents
				$this->writeOutput();
			}
		}
	}
}
