<?php
/**
 * @package     CSVI
 * @subpackage  J2Store
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - [year] RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace j2store\com_j2store\model\import;

defined('_JEXEC') or die;

/**
 * Variants import.
 *
 * @package     CSVI
 * @subpackage  J2Store
 * @since       7.7.0
 */
class Variant extends \RantaiImportEngine
{
	/**
	 * CSVI fields
	 *
	 * @var    \CsviHelperImportFields
	 * @since  7.7.0
	 */
	protected $fields;

	/**
	 * The addon helper
	 *
	 * @var    \Com_J2StoreHelperCom_J2Store
	 * @since  7.7.0
	 */
	protected $helper;

	/**
	 * Variants table
	 *
	 * @var    \J2StoreTableVariant
	 * @since  7.7.0
	 */
	private $variantsTable;

	/**
	 * Products table
	 *
	 * @var    \J2StoreTableProduct
	 * @since  7.8.0
	 */
	private $productTable;

	/**
	 * Start the menu import process.
	 *
	 * @return  bool  True on success | false on failure.
	 *
	 * @since   7.7.0
	 *
	 * @throws  \RuntimeException
	 * @throws  \InvalidArgumentException
	 * @throws  \UnexpectedValueException
	 */
	public function getStart()
	{
		// Process data
		foreach ($this->fields->getData() as $fields)
		{
			foreach ($fields as $name => $details)
			{
				$value = $details->value;

				switch ($name)
				{
					case 'parent_product_title':
					case 'parent_product_alias':
						$field     = str_replace('parent_product_', '', $name);
						$productId = $this->helper->getParentProductId($field, $value);
						$this->setState('product_id', $productId);
						break;
					case 'parent_product_id':
						$this->setState('product_id', $value);
						break;
					case 'shipping':
					case 'is_master':
					case 'use_store_config_min_out_qty':
					case 'use_store_config_min_sale_qty':
					case 'use_store_config_max_sale_qty':
					case 'use_store_config_notify_qty':
					case 'allow_backorder':
					case 'isdefault_variant':
						switch (strtolower($value))
						{
							case 'n':
							case 'no':
							case '0':
								$value = 0;
								break;
							default:
								$value = 1;
								break;
						}

						$this->setState($name, $value);
						break;
					default:
						$this->setState($name, $value);
						break;
				}
			}
		}

		$sku = $this->getState('sku', false);

		if (!$sku)
		{
			$this->loaded = false;
			$this->log->addStats('skipped', 'COM_CSVI_NO_PRODUCT_SKU_FOUND');
		}
		else
		{
			$this->loaded = true;
			$variantId    = $this->helper->getVariantId($sku);

			if ($variantId)
			{
				$this->setState('j2store_variant_id', $variantId);
			}

			if ($variantId && $this->variantsTable->load($this->getState('j2store_variant_id', 0)))
			{
				if (!$this->template->get('overwrite_existing_data'))
				{
					$this->log->add(\JText::sprintf('COM_FIELDS_WARNING_OVERWRITING_SET_TO_NO', $this->getState('sku')), false);
					$this->loaded = false;
				}
			}
		}

		return true;
	}

	/**
	 * Process a record.
	 *
	 * @return  bool  Returns true if all is OK | Returns false if no path or menu ID can be found.
	 *
	 * @since   7.7.0
	 *
	 * @throws  \RuntimeException
	 * @throws  \InvalidArgumentException
	 * @throws  \UnexpectedValueException
	 */
	public function getProcessRecord()
	{
		if (!$this->loaded)
		{
			return false;
		}

		if (!$this->getState('j2store_variant_id', false) && $this->template->get('ignore_non_exist'))
		{
			$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_IGNORE_NEW', $this->getState('sku', '')));
		}
		else
		{
			if (!$this->getState('product_id', false) || ($this->getState('product_id', false)
					&& !$this->productTable->load($this->getState('product_id', false))))
			{
				$this->log->add('Parent Product not found');
				$this->log->AddStats('skipped', \JText::_('COM_CSVI_NO_PARENT_PRODUCT_FOUND'));
			}
			else
			{
				if (!$this->getState('pricing_calculator', false))
				{
					$this->setState('pricing_calculator', 'standard');
				}

				if (!$this->getState('is_master', false))
				{
					$this->setState('is_master', 0);
				}

				$this->variantsTable->bind($this->state);
				$this->variantsTable->check();

				try
				{
					$this->variantsTable->store();
					$this->log->add('Variants added successfully', false);
					$this->setState('variant_id', $this->variantsTable->j2store_variant_id);
					$this->setState('product_id', $this->variantsTable->product_id);
					$this->helper->storeProductQuantity($this->variantsTable->j2store_variant_id, $this->getState('quantity', 0));

					if($this->getState('option_names', false) && $this->getState('option_values', false))
					{
						$this->updateVariantOptionsandValues();
					}
				}
				catch (\Exception $e)
				{
					$this->log->add('Cannot add Variants. Error: ' . $e->getMessage(), false);
					$this->log->addStats('incorrect', $e->getMessage());

					return false;
				}
			}
		}

		return true;
	}

	/**
	 * Process variants options and values
	 *
	 * @return  mixed false if needed fields are not set.
	 *
	 * @since   7.7.0
	 */
	private function updateVariantOptionsandValues()
	{
		$variantId    = $this->getState('variant_id', false);
		$productId    = $this->getState('product_id', false);
		$optionNames  = explode('~', $this->getState('option_names', false));
		$optionValues = explode('~', $this->getState('option_values', false));
		$optionType   = explode('~', $this->getState('option_types', false));

		if (!$optionNames || !$optionValues || !$optionType)
		{
			$this->log->add('Needed option name or value or type missing', false);

			return false;
		}

		if (!$variantId)
		{
			return false;
		}

		if (count($optionNames) === count($optionValues))
		{
			$variantOptions = array();

			foreach ($optionNames as $key => $optionName)
			{
				// Get the id of the option name and type
				$query = $this->db->getQuery(true)
					->select($this->db->quoteName('j2store_option_id'))
					->from($this->db->quoteName('#__j2store_options'))
					->where($this->db->quoteName('option_name') . ' = ' . $this->db->quote($optionName))
					->where($this->db->quoteName('type') . ' = ' . $this->db->quote($optionType[$key]));

				$this->log->add('Find the option name id');
				$this->db->setQuery($query);
				$optionNameId = $this->db->loadResult();

				if (!$optionNameId)
				{
					$query->clear()
						->insert($this->db->quoteName('#__j2store_options'))
						->columns($this->db->quoteName('option_name') . ', ' . $this->db->quoteName('type'))
						->values($this->db->quote($optionName) . ', ' . $this->db->quote($optionType[$key]));
					$this->log->add('No option name found so insert a new one');
					$this->db->setQuery($query)->execute();
					$optionNameId = $this->db->insertid();
				}

				$query = $this->db->getQuery(true)
					->select($this->db->quoteName('j2store_optionvalue_id'))
					->from($this->db->quoteName('#__j2store_optionvalues'))
					->where($this->db->quoteName('option_id') . ' = ' . (int) $optionNameId)
					->where($this->db->quoteName('optionvalue_name') . ' = ' . $this->db->quote($optionValues[$key]));

				$this->log->add('Find the option value id');
				$this->db->setQuery($query);
				$optionValueId = $this->db->loadResult();

				if (!$optionValueId)
				{
					$query->clear()
						->insert($this->db->quoteName('#__j2store_optionvalues'))
						->columns($this->db->quoteName('optionvalue_name') . ', ' . $this->db->quoteName('option_id'))
						->values($this->db->quote($optionValues[$key]) . ', ' . (int) $optionNameId);
					$this->log->add('Insert new option value id');
					$this->db->setQuery($query)->execute();
					$optionValueId = $this->db->insertid();
				}

				$query->clear()
					->select($this->db->quoteName('j2store_productoption_id'))
					->from($this->db->quoteName('#__j2store_product_options'))
					->where($this->db->quoteName('option_id') . ' = ' . (int) $optionNameId)
					->where($this->db->quoteName('product_id') . ' = ' . (int) $productId);
				$this->log->add('Find the product option id');
				$this->db->setQuery($query);
				$productOptionNameId = $this->db->loadResult();

				if (!$productOptionNameId)
				{
					$query->clear()
						->insert($this->db->quoteName('#__j2store_product_options'))
						->columns($this->db->quoteName('option_id') . ', ' . $this->db->quoteName('product_id'))
						->values((int) $optionNameId . ', ' . (int) $productId);
					$this->log->add('Insert a new product option');
					$this->db->setQuery($query)->execute();
					$productOptionNameId = $this->db->insertid();
				}

				$query->clear()
					->select($this->db->quoteName('j2store_product_optionvalue_id'))
					->from($this->db->quoteName('#__j2store_product_optionvalues'))
					->where($this->db->quoteName('productoption_id') . ' = ' . (int) $productOptionNameId)
					->where($this->db->quoteName('optionvalue_id') . ' = ' . (int) $optionValueId);
				$this->log->add('Find the product option value id');
				$this->db->setQuery($query);
				$productOptionValueId = $this->db->loadResult();

				if (!$productOptionValueId)
				{
					$query->clear()
						->insert($this->db->quoteName('#__j2store_product_optionvalues'))
						->columns($this->db->quoteName('productoption_id') . ', ' . $this->db->quoteName('optionvalue_id'))
						->values((int) $productOptionNameId . ', ' . (int) $optionValueId);
					$this->log->add('Insert a new product option value');
					$this->db->setQuery($query)->execute();
					$productOptionValueId = $this->db->insertid();
				}

				$variantOptions[] = $productOptionValueId;

			}

			// Insert the new combination for a variant
			$query = $this->db->getQuery(true)
				->delete($this->db->quoteName('#__j2store_product_variant_optionvalues'))
				->where($this->db->quoteName('variant_id') . ' = ' . (int) $variantId);
			$this->log->add('Delete the variant entries before inserting');
			$this->db->setQuery($query)->execute();

			$combResult = implode(',', $variantOptions);

			$query->clear()
				->select($this->db->quoteName('product_optionvalue_ids'))
				->from($this->db->quoteName('#__j2store_product_variant_optionvalues'))
				->where($this->db->quoteName('variant_id') . ' = ' . (int) $variantId)
				->where($this->db->quoteName('product_optionvalue_ids') . ' = ' . $this->db->quote($combResult));
			$this->log->add('Find the product option value id');
			$this->db->setQuery($query);
			$result = $this->db->loadResult();

			if (!$result)
			{
				$query->clear()
					->insert($this->db->quoteName('#__j2store_product_variant_optionvalues'))
					->columns($this->db->quoteName('variant_id') . ', ' . $this->db->quoteName('product_optionvalue_ids'))
					->values((int) $variantId . ', ' . $this->db->quote($combResult));
				$this->log->add('Insert new variant combination');
				$this->db->setQuery($query)->execute();
			}
			else
			{
				$this->log->add('Variant with combination already exists');
			}
		}
		else
		{
			$this->log->add('Option name and Option values are not of same count');
			$this->log->addStats('error', 'COM_CSVI_OPTION_COUNT_NOT_SAME');
		}

		return true;
	}

	/**
	 * Load the necessary tables.
	 *
	 * @return  void.
	 *
	 * @since   7.7.0
	 */
	public function loadTables()
	{
		$this->variantsTable = $this->getTable('Variant');
		$this->productTable = $this->getTable('Product');
	}

	/**
	 * Clear the loaded tables.
	 *
	 * @return  void.
	 *
	 * @since   7.7.0
	 */
	public function clearTables()
	{
		$this->variantsTable->reset();
		$this->productTable->reset();
	}
}
